/**
 * Created by Connor Weeks on 3/18/2017.
 */

// the number of seconds remaining in the parking timer
var secondsRemaining = 0;

// the ID of the interval controlling the timer
var timerIntervalId;

function updateTimer() {

    var hours = Math.floor(secondsRemaining / 3600);
    var minutes = Math.floor((secondsRemaining % 3600) / 60);
    var seconds = secondsRemaining % 60;

    hours = hours < 10 ? '0' + hours : hours.toString();
    minutes = minutes < 10 ? '0' + minutes : minutes.toString();
    seconds = seconds < 10 ? '0' + seconds : seconds.toString();

    $('#hours').text(hours);
    $('#minutes').text(minutes);
    $('#seconds').text(seconds);
}

function startParkingTimer(hours, socket) {
    secondsRemaining = hours * 3600;
    updateTimer();
    timerIntervalId = setInterval(function () {
        secondsRemaining--;
        updateTimer();
        if (secondsRemaining <= 0) {
            stopParking(socket);
        }
    }, 1000);
}

function stopParking(socket) {
    // stop the timer
    clearInterval(timerIntervalId);

    // hide the timer and stop parking button
    $('#stop-parking').hide();
    $('#parking-timer').hide();
    $('#start-parking').text('Start Parking');

    // reset the timer and allow users to select new objects
    secondsRemaining = 0;
    $('#parking-locations').prop('disabled', false);

    // emit an event to the server to signal that a car has left
    socket.emit('stop-parking', {
        locationId: selected.id
    });

    // allow other locations and zones to be selected
    currentlyParking = false;
}
/**
 * This function exists in each *.js file but functions slightly differently.
 * In this case, this function displays data about the given zone or location
 * using getData.
 * @param id the ID of teh zone or location to display.
 * @param type The type of the ID given (TYPE_ZONE or TYPE_LOCATION).
 */
function displayData(id, type) {
    // first get the data about either the zone or location
    var data = getData(id, type);
    if (data) {
        // determine individual pricing based on occupancy and emissions
        var emissionsBand = parseInt($('#emissions-band').val());
        var occupancyMultiplier = interpolate(0, 1, 1, data.occupancyMultiplier, data.fractionOccupied);
        var price = interpolate(1, 6, data.band1Price, data.band6Price, emissionsBand) * occupancyMultiplier;

        $('#zone-label').text('Zone: ' + data.zoneName);
        $('#price-label').text('Price: £' + price.toFixed(2) + ' / hr');
        $('#occupancy-label').text('Occupancy: ' + (data.fractionOccupied * 100).toFixed() + '%');
        $('#location-label').text('Location: ' + data.locationName);
        $('#parking-locations').val(data.locationId);
    } else {
        // display no values if nothing is selected
        $('#zone-label').text('Zone: None');
        $('#price-label').text('Price: None');
        $('#occupancy-label').text('Occupancy: None');
        $('#location-label').text('Location: None');
        $('#parking-locations').val('');
    }

    $('#start-parking').prop('disabled', type != TYPE_LOCATION || !selected.id)
}

$(document).ready(function () {

    // connect to the server through socket.io
    var socket = io();
    setupUpdateEvent(socket);

    // respond to altered emission band
    $('#emissions-band').change(function () {
        if (selected.id) {
            displayData(selected.id, selected.type);
        }
    });

    // when the parking zone is changed in the select field,
    // alter teh selected zone on the map
    $('#parking-locations').change(function () {
        var locationId = $(this).val();
        if (locationData[locationId]) {
            setSelected(locationId, TYPE_LOCATION);
        }

    });

    // handle when a user clicks the 'start parking' button
    $('#start-parking').click(function () {
        var numHours = parseInt($('#num-hours').val());
        if (!numHours || numHours < 0) {
            return;
        }

        if (currentlyParking) {
            // top off the time if currently parking
            secondsRemaining += numHours * 3600;
            updateTimer();
        } else {
            currentlyParking = true;
            // start and display the parking timer
            startParkingTimer(numHours, socket);
            $('#parking-timer').show();

            // change the text of the start parking button to 'top off'
            $(this).text('Top Off');

            // display the stop parking button and prevent the user from selecting other locations
            $('#stop-parking').show();
            $('#parking-locations').prop('disabled', true);

            // emit an event to the server to signal the newly parked car
            socket.emit('start-parking', {
                locationId: selected.id
            });
        }
    });

    // handle when a user clicks the 'stop parking' button
    var stopParkingButton = $('#stop-parking');
    stopParkingButton.click(function () {
        stopParking(socket);
    });

    // this is just used as a delimiter when adding new points to the map
    // it serves no purpose to the user
    document.addEventListener('keyup', function(event) {
        if ((event.keyCode || event.which) == 32) {
            console.log('New');
        }
    });

    // handle a change of value in the show pay and display checkbox
    var showPdCheckbox = $('#show-pd');
    showPdCheckbox.change(function() {
        // reset the map style
        showPd = $(this).is(":checked");
        map.data.setStyle(setStyle);
    });

    // handle when a user changes the highlighting checkbox
    $('#keep-highlight').change(function () {
        // reset the map style
        highlightWhenZoomed = $(this).is(':checked');
        map.data.setStyle(setStyle);
    });

    // handle mouse events to determine if the mouse is pressed
    $(document).mousedown(function() {
        mouseDown = true;
    });
    $(document).mouseup(function() {
        mouseDown = false;
    });

    // set the initial value of whether or not to show pay and display machines
    showPd = showPdCheckbox.is(":checked");

    stopParkingButton.hide();
});
